# currently we use -bexpall to export all the symbols, but that causes functions
# like memcpy, memset, etc that are statically linked from libc to be exported
# as well. We can't disable -bexpall until we have every package that links to
# it to require a version which doesn't export these functions, but that gets us
# in to a catch-22. So we do it in stages:
#
# 1. Update lua to use -bexpall, but remove the symbols from the export file,
#    so anyone linking to lua won't use those symbols
# 2. Build lua
# 3. Update all dependants with Requires: liblua >= (fixed version)
# 4. Rebuild all dependants
# 5. Update lua to not use -bexpall and rebuild
%bcond_without bexpall_compat


%define major 5
%define minor 3
%define fix   4

Name: lua
Version: %{major}.%{minor}.%{fix}
Release: 4
License: MIT
Summary: LUA programming language
Url: http://www.lua.org

BuildRequires: readline-devel
BuildRequires: pase-build-tools

Source0: http://www.lua.org/ftp/lua-%{version}.tar.gz


%description
Lua is a powerful, efficient, lightweight, embeddable scripting language. It
supports procedural programming, object-oriented programming, functional
programming, data-driven programming, and data description.

Lua combines simple procedural syntax with powerful data description constructs
based on associative arrays and extensible semantics. Lua is dynamically typed,
runs by interpreting bytecode with a register-based virtual machine, and has
automatic memory management with incremental garbage collection, making it ideal
for configuration, scripting, and rapid prototyping.

%package -n liblua5_3
Summary: file library

%description -n liblua5_3
Lua shared library.

%package devel
Summary: file development files
Requires: liblua5_3 = %{version}-%{release}

%description devel
Lua development files.

%prep
%setup -q

%build


cd src

%define ldflags -Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib

%make_build all \
    CFLAGS="$CFLAGS -DLUA_USE_POSIX -DLUA_USE_DLOPEN -DLUA_USE_READLINE -DLUA_COMPAT_MODULE" \
    SYSLIBS="-ldl -lreadline" \
    SYSLDFLAGS="%{ldflags}"

%define soname liblua.so.%{major}.%{minor}

# first, build shared library that doesn't use -bexpall, the shr_64.imp won't
# list the offending symbols this way so dependants won't can't link to them
static-to-shared %{soname} liblua.a %{ldflags} -ldl -lm

gcc $CFLAGS -std=gnu99 -o lua %{ldflags} lua.o -L. -llua -lm -ldl -lreadline
gcc $CFLAGS -std=gnu99 -o luac %{ldflags} luac.o -L. -llua -lm -ldl -lreadline

%if %{with bexpall_compat}
echo "----------- Building with -bexpall compatibility --------------------"
# now build the shared library the old way that uses -bexpall
gcc -shared -o shr_64.o `ar t liblua.a` %{ldflags} -Wl,-bexpall -ldl -lm

# Mark it unlinkable, so the import file is used
/QOpenSys/usr/bin/strip -e shr_64.o

# Put back in the -bexpall shr_64.o for compatibility until we've rebuilt
# all the dependant packages.
ar crlo %{soname} shr_64.o
%endif

cd ..

%install

%make_build install \
  INSTALL_TOP=%{buildroot}/%{_prefix} \
  INSTALL='cp -p' \
  INSTALL_EXEC='$(INSTALL)' \
  INSTALL_DATA='$(INSTALL)' \
  INSTALL_MAN='%{buildroot}/%{_mandir}/man1' \
  # end

mkdir -p %{buildroot}/%{_libdir}/pkgconfig
cat <<EOF > %{buildroot}/%{_libdir}/pkgconfig/lua.pc
prefix=%{_prefix}
exec_prefix=%{_exec_prefix}
libdir=%{_libdir}
includedir=%{_includedir}

Name: %{name}
Version: %{version}
Description: %{summary}
URL: %{url}
Libs: -L\${libdir} -llua
Cflags: -I\${includedir}
EOF

cp -p src/liblua.so.5.3 %{buildroot}/%{_libdir}
ln -s liblua.so.5.3 %{buildroot}/%{_libdir}/liblua.so

rm %{buildroot}/%{_libdir}/liblua.a

%files
%defattr(-, qsys, *none)

%attr(755, -, -) %{_bindir}/lua
%attr(755, -, -) %{_bindir}/luac

%{_datadir}/lua/5.3

%{_mandir}/man1/luac.1*
%{_mandir}/man1/lua.1*

%files -n liblua5_3
%defattr(-, qsys, *none)

%{_libdir}/liblua.so.*

%files devel
%defattr(-, qsys, *none)

%{_libdir}/liblua.so
%{_libdir}/pkgconfig/lua.pc

%{_includedir}/lua.h
%{_includedir}/luaconf.h
%{_includedir}/lua.hpp
%{_includedir}/lauxlib.h
%{_includedir}/lualib.h


%changelog
* Thu Nov 4 2021 Jason Hansen <stymie@us.ibm.com> - 5.3.4
- Intra-package Requires need version and release

* Wed May 27 2020 Kevin Adler <kadler@us.ibm.com> - 5.3.4
- Use generic paths for man and info pages to allow for compression

* Mon Dec 16 2019 Kevin Adler <kadler@us.ibm.com> - 5.3.4
- Mark shr_64.o non-linkable even when compat enabled

* Wed Nov 27 2019 Kevin Adler <kadler@us.ibm.com> - 5.3.4
- Fix build to not list memcpy, math funcs in liblua.so import file
- Keep liblua shr_64.o which exports these funcs for compat, though

* Tue Mar 26 2019 Kevin Adler <kadler@us.ibm.com> - 5.3.4
- ibm-build-tools -> pase-build-tools

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Add missing BuildRequires for readline-devel

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Mass replace svr4_shlib.sh -> svr4-shrlib

* Wed Dec 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Remove 64-bit GCC script workarounds
- OBJECT_MODE now always set by build and install macros

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- Remove unneeded Provides/Requires lines

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- Use auto dependency resolution instead of requiring pase-libs

* Tue Nov 07 2017 - Kevin Adler <kadler@us.ibm.com>
- Fix license
